-- Agent based modeling
-- Author: Pedro Andrade & Tiago Carneiro

-------------------------------------------
-- GLOBAL
-------------------------------------------

EMPTY = 0
FULL = 1
-------------------------------------------
-- MODEL 
-------------------------------------------

--# space
cs = CellularSpace {
	xdim = 30,
	ydim = 30
}

cs:createNeighborhood()

forEachCell(cs, function(cell)
	cell.state = EMPTY
end)

--# behavior

singleFooAgent = Agent {
	execute = function(self)
		cell = self:getCell():getNeighborhood():sample()
		if cell.state == EMPTY then
			self:getCell().state = EMPTY
			self:move(cell)
			cell.state = FULL
		end
	end
}

--# time 

t = Timer {
	Event{action = singleFooAgent},
	Event{action = cs}
}

--# environment

e = Environment {
	cs,
	singleFooAgent,
	t
}

e:createPlacement{strategy = "random"}

-------------------------------------------
-- SIMULATION
-------------------------------------------

leg = Legend {
	grouping = "uniquevalue",
	colorBar = {
		{value = EMPTY, color = "white"},
		{value = FULL, color = "black"}
	}
}

Observer {
	subject = cs,
	attributes = {"state"},
	legends = {leg}
}

e:execute(1000)

